#!/usr/bin/perl -w
#
# This program is used to run strobe-classb.c.  It is called from
# relay-check.pl as well as being able to be run as a standalone
# program by editing config.pl for by making a config-$config.pl
# and running as 'strobe-classb.pl $config'.
#
# See the file COPYRIGHT for copyright information.
# (c) David Luyer, 1998.

use strict;
my $config = "config.pl";
$config = "config-$ARGV[0].pl"
  if(defined $ARGV[0]);
$config = "main process"
  if(defined $main::sub);
open(IMAX, "</proc/sys/kernel/inode-max");
die "inode-max too low - I recommend 32768" if(int(<IMAX>) < 20000);
close(IMAX);
open(FMAX, "</proc/sys/kernel/file-max");
die "file-max too low - I recommend 16384" if(int(<FMAX>) < 10000);
close(FMAX);
my $kernel = `uname -r`;
die "kernel version too low - should be 2.0.35 release at least"
  if($kernel !~ /^(2\.(0\.(3[5-9]|[4-9][0-9])|[1-9])|[3-9])/);
undef $kernel;
require "$config"
  if($config ne "main process");
die "$config must define \@config::ports"
  if(!defined @config::ports);
die "$config must define \@config::classbs"
  if(!defined @config::classbs);
die "$config must define \$config::dispose"
  if(!defined $config::dispose);

sub make_strobe_classb {
  my ($net, $port) = @_;

  print STDERR "Building strobe-classb-$net-$port...";
  system("/usr/bin/gcc","-c","-Wall","-ansi","-pedantic","-O2","-fno-builtin",
         "-fomit-frame-pointer","-malign-loops=2","-malign-jumps=2",
         "-DPORT=$port", "-DCLASSB=$net"."L", "strobe-classb.c", "-o",
         "strobe-classb-$net-$port.o");
  system("/usr/bin/ld","-sNx","-o","strobe-classb-$net-$port",
         "strobe-classb-$net-$port.o");
  unlink("strobe-classb-$net-$port.o");
  if( -x "strobe-classb-$net-$port" ) {
    print STDERR "done.\n";
  } else {
    print STDERR "failed.\n";
    die "compile failed";
  }
}

my $classb;
my $i = 0;
@main::hosts = ()
  if(defined $main::sub);
for $classb (@config::classbs) {
  die "Invalid class B network: $classb"
    if($classb !~ /^(\d+)\.(\d+)$/);
  my $net = $2 * 256 + $1;
  my $port;
  for $port (@config::ports) {
    my $oport = $port;
    $port = getservbyname($port, 'tcp')
      if($port !~ /^(\d+)$/);
    die "Invalid port: $oport"
      if($port !~ /^(\d+)$/);
    $port *= 256;
    make_strobe_classb($net, $port)
      if( ! -x "strobe-classb-$net-$port" );
    $i++;
    open(STROBE_CLASSB, "-|") || exec ("strobe-classb-$net-$port", "$i");
    while(<STROBE_CLASSB>) {
      die "Misc error from strobe-classb"
        if($_ eq "XXXX\n");
      die "Bad strobe-classb output $_"
        if(!/^(..)(..)\n$/);
      my ($i1, $i2) = (hex $1, hex $2);
      print (($_ = $classb.".$i1.$i2:$oport")."\n");
      push @main::hosts, $_
        if (defined $main::sub);
    }
    close(STROBE_CLASSB);
    unlink("strobe-classb-$net-$port")
      if($config::dispose == 1);
  }
}
# Since this may be 'required' by something like check-relay.pl
1;
